/**
    Martin Flodn
    Summer 1996
    File name: gradhess.prg

    This file contains procedures to approximate the gradient and
    Hessian of a function
**/

/** Calculate gradient, see Cooley, p. 45 **/
proc gradient(f,x0,dh);
    local f: proc;
    local n,dhi,i,g,h;

    n = rows(x0);

    g = zeros(n,1);

    i = 1;
    do while (i <= n);
        dhi = zeros(n,1);
        dhi[i] = maxc((dh*abs(x0[i]))|1e-6);
        g[i] = (f(x0 + dhi) - f(x0 - dhi)) / (2*dhi[i]);
        i = i + 1;
    endo;

    retp(g);
endp;

/** Calculate Hessian, see Cooley, pp. 45-46 **/
proc hessian(f,x0,dh);
    local f: proc;
    local n,dhi,dhj,dh0,dh1,i,j,g,h;

    n = rows(x0);

    h = zeros(n,n);

    i = 1;
    do while (i <= n);
        dh0 = maxc((dh * abs(x0[i]))|(dh/1000));
        dhi = zeros(n,1);
        dhi[i] = dh0;
        j = 1;
        do while (j <= n);
            dh1 = maxc((dh * abs(x0[j]))|(dh/1000));
            dhj = zeros(n,1);
            dhj[j] = dh1;
            h[i,j] = (f(x0 + dhi + dhj) + f(x0 - dhi - dhj)
                 - f(x0 + dhi - dhj) - f(x0 - dhi + dhj)) / (4*dh0*dh1);
            j = j + 1;
        endo;
        i = i + 1;
    endo;

    retp(h);
endp;

